package org.xdv.clx.base;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.xdv.clx.exec.ClxExecutionContext;
import org.xdv.clx.expr.ClxRules;
import org.xdv.common.XdvErrorLevelSet;
import org.xdv.common.XdvValidationError;
import org.xdv.common.XdvValidationException;
import org.xdv.common.XdvValidator;
import org.xdv.xpath.XPathEngine;

/**
 * CLX validator - validates XML document agains rules defined in CLiXML language.
 * <p>
 * It gives several ways to validate XML document:
 * <ul>
 *  <li>{@link #validate(Object)} - general validation, you get simple answer: 
 *      <i>valid</i>/<i>not valid</i>,
 *  <li>{@link #validateSimple(Object)} - simple validation, you get one reported error. 
 *      Usually it will be the first reported error, but it depends on implementation,
 *  <li>{@link #validateDeep(Object)} - deep validation, you get list of all reported errors.
 * </ul>
 * Important notes about this implementation:
 * <ul>
 *  <li>Implementation is fully compatible with {@link XdvValidator}.
 *  <li>In general <i>deep validation</i> is much slower than <i>simple validation</i> and 
 *      <i>general validation</i>. But if source XML document is valid then all validation
 *      methods will finish in the same time.
 *  <li><i>Simple validation</i> and <i>general validation</i> offers exacly same performance.
 *  <li>Each validation type has an extended version where user can specify to validate only
 *      against CLX rules from given error-level set. 
 *      For instance: {@link #validateDeep(Object, XdvErrorLevelSet)}.
 *  <li>Type of XML source depends only on {@link org.xdv.xpath.XPathEngine} used to create CLX rules.
 *  <li>Instances should not be created directly but rather through {@link ClxValidatorFactory}.
 * </ul>
 * </p>
 *
 * @author Tomasz Kokoszka
 */
public class ClxValidator implements XdvValidator {
    
    /** XPath engine to execute prepared XPath expressions. 
     * It must be the same engine instance that was used to create CLX rules */
    protected final XPathEngine xpathEngine;
    
    /** CLX rule set */
    protected final ClxRules rules;
    
    /**
     * Creates new validator.
     * @param xpathEngine engine used to execute prepared XPath expressions.
     *        According to {@link XPathEngine} deifnition it must be the same instance 
     *        that was used to create CLX rules
     * @param rules rule set
     */
    public ClxValidator(XPathEngine xpathEngine, ClxRules rules) {
        this.xpathEngine = xpathEngine;
        this.rules = rules;
    }    
    
    /**
     * Returns rule set
     * @return rule set; !=null
     */
    public ClxRules getRules() {
        return this.rules;
    }        
    
    public Class getDocumentClass() {
        return xpathEngine.getDocumentClass();
    }
    
    /**
     * Checks XML document type - if document is not an instance of {@link #getDocumentClass()} 
     * then a {@link XdvValidationException} is thrown.
     * @param document XML document
     * @throws XdvValidationException document type is not valid for this validator
     */
    private void checkDocumentClass(Object document) 
    throws XdvValidationException {
        if (! getDocumentClass().isInstance(document)) {
            throw new XdvValidationException(
                    "Document type " + ((document == null)?"null":document.getClass().getName()) + 
                    " is not supported. " + getDocumentClass().getName() + " required"); 
        }        
    }

    public boolean validate(Object document, XdvErrorLevelSet errorSet) 
    throws XdvValidationException {      
        checkDocumentClass(document);
        
        final ClxExecutionContext cntx = new ClxExecutionContext(xpathEngine, document);
        return this.rules.execute(cntx, errorSet);                
    }

    public XdvValidationError validateSimple(Object document, XdvErrorLevelSet errorSet) 
    throws XdvValidationException {
        checkDocumentClass(document);

        final ClxExecutionContext cntx = new ClxExecutionContext(xpathEngine, document);
        return this.rules.executeSimple(cntx, errorSet);
    }
    
    public Collection validateDeep(Object document, XdvErrorLevelSet errorSet) 
    throws XdvValidationException {
        checkDocumentClass(document);
        
        final ClxExecutionContext cntx = new ClxExecutionContext(xpathEngine, document);
        List errors = new ArrayList();
        this.rules.executeDeep(cntx, errors, errorSet);
        return errors;
    }
    
    public boolean validate(Object document) 
    throws XdvValidationException {                
        return validate(document, XdvErrorLevelSet.ALL);
    }

    public XdvValidationError validateSimple(Object document) 
    throws XdvValidationException {
        return validateSimple(document, XdvErrorLevelSet.ALL);
    }
    
    public Collection validateDeep(Object document) 
    throws XdvValidationException {
        return validateDeep(document, XdvErrorLevelSet.ALL);
    }
       
}
