package org.xdv.clx.expr;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

import org.xdv.clx.extension.ClxExtMacroCall;
import org.xdv.clx.operator.ClxOperatorCall;

/**
 * MACRO DEFINITIONS - CLX element implementation.
 * <br>
 * It is a set of macro definitions.
 * 
 * @see <a href="http://www.clixml.org/clix/1.0/docs/html/#element_macroDefinitions_Link037184F0">CLiXML - macroDefinitions</a>
 * @see org.xdv.clx.base.ClxConstants#TAG_MACRODEFINITIONS
 * @author Tomasz Kokoszka
 */
public class ClxMacroDefinitions {

    /** Header, null means no header */
    private final ClxHeader header;
    
    /** Map of all macros in this set: macro's name-&gt;macro's definition 
     * ({@link String}-&gt;{@link ClxMacro}).
     */
    private final Map macros;
    
    /**
     * All macro's calls from all forumals inside this macro-set.
     * Collection of {@link ClxExtMacroCall}; !=null.
     */
    private Collection macroCalls = new ArrayList(0);
    
    /**
     * All operator's calls from all formulas inside this macro-set.
     * Collection of {@link ClxOperatorCall}; !=null.
     */
    private Collection operatorCalls = new ArrayList(0);
        
    /**
     * Create an empty macro-set.
     * @param header header, can be null
     */
    public ClxMacroDefinitions(ClxHeader header) {
        this.header = header;
        this.macros = new LinkedHashMap();
    }

    /** 
     * Returns header.
     * @return header, null means there is no header
     */
    public ClxHeader getHeader() {
        return this.header;        
    }
    
    /**
     * Returns macro's defintion.
     * @param macroName macro's name
     * @return macro's definitions for given macro's name or <code>null</code> if such macro
     *         is not in this set
     */
    public ClxMacro getMacro(String macroName) {
        return (ClxMacro) macros.get(macroName);
    }
    
    /**
     * Returns all macros from this set.
     * @return collection of {@link ClxMacro}; !=null
     */
    public Collection getMacros() {
        return this.macros.values();
    }
    
    /**
     * Returns all macro's names from this set.
     * @return collection of {@link String}
     */
    public Set getMacroNames() {
        return this.macros.keySet();
    }
    
    /**
     * Returns macro's map.
     * It returns directly structure used internally by instance - do not modify it.
     * @return macro's name-&gt;macro's definition ({@link String}-&gt;{@link ClxMacro}); !=null
     */
    public Map getMacroMap() {
        return this.macros;
    }
    
    /** 
     * Adds macro to this set.
     * @param macro macro to add
     * @return macro that was before in this set under same name, or <code>null</code> if set
     *         didn't have a macro with same name before
     */
    public ClxMacro addMacro(ClxMacro macro) {
        return (ClxMacro) macros.put(macro.getName(), macro);
    }
    
    /**
     * Adds all macros from collection to this set.
     * @param macroCol collections of macros to add (collection of {@link ClxMacro})
     * @return macro that was before in this set under same name, or <code>null</code> if set
     *         didn't have a macro with same name before
     */
    public ClxMacro addMacroAll(Collection macroCol) {
        ClxMacro duplicate = null;
        for(Iterator i = macroCol.iterator(); i.hasNext(); ) {
            ClxMacro old = addMacro((ClxMacro) i.next());
            if (old != null) {
                duplicate = old;
            }
        }
        return duplicate;
    }        
    
    /**
     * Returns all macro's calls from all forumals inside this macro-set.
     * @return collection of {@link ClxExtMacroCall}; !=null
     */
    public Collection getMacroCalls() {
        return macroCalls;
    }

    /**
     * Sets internal colection of macro's calls.
     * @param macroCalls collection of {@link ClxExtMacroCall}
     */
    public void setMacroCalls(Collection macroCalls) {
        this.macroCalls = macroCalls;
    }

    /**
     * Returns all operator's calls from all forumals inside this macro-set.
     * @return collection of {@link ClxOperatorCall}; !=null
     */
    public Collection getOperatorCalls() {
        return operatorCalls;
    }

    /**
     * Sets internal colection of operator's calls.
     * @param operatorCalls collection of {@link ClxOperatorCall}
     */
    public void setOperatorCalls(Collection operatorCalls) {
        this.operatorCalls = operatorCalls;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("{");
        for(Iterator i = macros.keySet().iterator(); i.hasNext(); ) {
            sb.append(i.next());
            if (i.hasNext()) {
                sb.append(" ");
            }            
        }
        sb.append("}");
        return sb.toString();
    }
    
}
