package org.xdv.clx.extension;

import java.util.Map;

import org.xdv.clx.builder.ClxBuilderException;
import org.xdv.clx.exec.ClxExecutionContext;
import org.xdv.clx.expr.ClxFormula;
import org.xdv.clx.expr.ClxMacro;
import org.xdv.common.XdvValidationException;

/**
 * Post prepared MACRO CALL - CLX extended element implementation.
 * <p>
 * This call can be created before definition of called macro is known. But before first execution
 * one must <i>post prepare</i> this call by simply specifying required macro definition.
 * It means that it support recurent and forwarding calls, but requires <i>post preparation</i>.
 * </p>
 * 
 * @author Tomasz Kokoszka
 */
public class ClxExtMacroCallPostPrep extends ClxExtMacroCall {

    /** Macro name; !=null */
    private final String macroName;
    
    /** Formula from macro definition. Value <code>null</code> means that this call has not been
     * prepared to be executed yet. */
    private ClxFormula output = null;
    
    /**
     * Create post prepared macro call.
     * @param name macro name
     * @param params macro parameters: parameter name-&gt;expression to evaluate its value 
     *        ({@link String}-&gt;{@link org.xdv.xpath.XPathExpression})
     */
    public ClxExtMacroCallPostPrep(String name, Map params) {
        super(params);
        this.macroName = name;
    }
        
    public String getMacroName() {
        return this.macroName;
    }
    
    /**
     * Is call ready to execution?
     * Call is ready if has been <i>post prepared</i> (see {@link #postPrepare(ClxMacro)}).
     * @return value <code>true</code> if this call has been post prepared
     */
    public boolean isReady() {
        return this.output != null;
    }
    
    /**
     * Post prepares this macro call.
     * It checks parameters (parameters defined in call must be the same as in macro definition), 
     * then it gets formula from macro definition and stores in internal private attribute. 
     * <br>
     * After calling this method, {@link #isReady()} will return <code>true</code>.
     * @param macroDefinition macro definition; can be null
     * @return previous formula used by this call, <code>null</code> means that this call has not 
     *         been ready for executing before  
     * @throws ClxBuilderException macro definition is empty (<code>null</code>), call parameters
     *         do not match parameters required by macro
     */
    public ClxFormula postPrepare(ClxMacro macroDefinition) 
    throws ClxBuilderException {        
        if (macroDefinition == null) {
            throw new ClxBuilderException("Macro " + getMacroName() + " called but not defined");
        }        
        assert macroDefinition != null;
        
        if (! getParams().equals(macroDefinition.getParams())) {
            throw new ClxBuilderException("Parameters for call and definition are " +
                    "not equal for macro " + getMacroName());
        }
        
        ClxFormula old = output;
        output = macroDefinition.getOutput();
        return old;
    }    
    
    /**
     * Returns macro output - formula that will executed by this macro call.
     * <br>
     * This implementation simply returns a private attribute. 
     * If this call has not been <i>post prepared</i> for execution it throws 
     * {@link XdvValidationException}.      
     * @param cntx execution context
     * @return formula to execute; !=null
     * @throws XdvValidationException this call is not ready to execute - it has not been
     *         <i>post prepared</i>
     */
    protected ClxFormula getOutput(ClxExecutionContext cntx) 
    throws XdvValidationException {
        if (! isReady()) {
            throw new XdvValidationException("Macro call " + macroName + " has not been " +
                    "prepared for execution");
        }
        
        return output;
    }
    
    public String toString() {
        return "macroCall:postPrepare(" + macroName + ")" + super.toString() + "{" + output + "}";
    }

}
