package org.xdv.xpath;

import java.util.HashSet;
import java.util.Set;

/**
 * Set of convenient static methods related to XPath.
 * <br>
 * This class should never be instantiated.
 * 
 * @author Tomasz Kokoszka
 */
public class XPathUtil {
   
    /**
     * Returns XPath value type.
     * <br>
     * Conversion table is as follow:
     * <table>
     * <tr>
     *  <td align='center'><b>XPath type</b></td><td align='center'><b>Returned name</b></td>
     * </tr>
     * <tr><td>{@link XPathValue#TYPE_BOOLEAN}</td><td><code>boolean</code></td></tr>
     * <tr><td>{@link XPathValue#TYPE_NODESET}</td><td><code>node-set</code></td></tr>
     * <tr><td>{@link XPathValue#TYPE_NUMBER}</td><td><code>number</code></td></tr>
     * <tr><td>{@link XPathValue#TYPE_STRING}</td><td><code>string</code></td></tr>
     * <tr><td>{@link XPathValue#TYPE_OTHER}</td><td><code>other</code></td></tr>
     * <tr><td>any other</td><td><code>unknown(<i>&lt;valueType&gt;</i>)</code></td></tr>
     * </table>
     * @param valueType XPath value type ({@link XPathValue#getType()} - one 
     *        of {@link XPathValue#TYPE_BOOLEAN} etc.
     * @return XPath type name; !=null.
     */
    public static String getValueTypeName(int valueType) {
        switch (valueType) {
        case XPathValue.TYPE_BOOLEAN:
            return "boolean";
        case XPathValue.TYPE_NODESET:
            return "node-set";
        case XPathValue.TYPE_NUMBER:
            return "number";
        case XPathValue.TYPE_STRING:
            return "string";
        case XPathValue.TYPE_OTHER:
            return "other";
        default:
            return "unknown(" + Integer.toString(valueType) + ")";
        }
    }
    
    /**
     * Returns XPath value type - short-cut to {@link #getValueTypeName(int)}.
     * @param value XPath value
     * @return XPath type name; !=null.
     */
    public static String getValueTypeName(XPathValue value) {
        return getValueTypeName(value.getType());
    }
    
    /**
     * Extracts variable names from xpath expression.
     * This method is guaranteed to work correctly only for legal, correct XPath expression. 
     * For invalid epxression method behaviour is not defined - may return any values 
     * (but still !=null). 
     * @param xpath XPath expression. It should be a correct expression otherwise this method may 
     *        return unexpected (incorrect) result.
     * @return set of {@link String}; !=null
     */
    public static Set extractVariableNames(String xpath) {
        final Set vars = new HashSet(); //set of String
                
        int token = 0; //0-other, 1-string ", 2-string ', 3-variable name
        StringBuffer varName = new StringBuffer();
        
        for(int i = 0; i < xpath.length(); i++) {
            final char c = xpath.charAt(i);
            if (token == 0) { //other
                if (c == '\\') { //skip next char 
                    i++;
                } else if (c == '\'') {
                    token = 2;
                } else if (c == '"') {
                    token = 1;
                } else if (c == '$') {
                    token = 3;
                }
            } else if (token == 1) { //string "
                if (c == '\\') { //skip next char
                    i++;
                } else if (c == '"') {
                    token = 0;
                }
            } else if (token == 2) { //string '
                if (c == '\\') { //skip next char
                    i++;
                } else if (c == '\'') {
                    token = 0;
                }
            } else if (token == 3) { //variable name
                if (Character.isLetterOrDigit(c) || c == '-' || c == '_' || c == '.') {
                    //those are all acceptable characters that may appear in variable name
                    varName.append(c);
                } else {
                    vars.add(varName.toString());
                    varName = new StringBuffer();
                    token = 0;
                }
            } else {
                assert false;
            }
        }
        if (token == 3) {
            vars.add(varName.toString());
        }        
        
        return vars;
    }
    
    /**
     * Private constructor to avoid creating instances.
     */
    private XPathUtil() {
        assert false;
    }

}
