package org.xdv.xpath.jaxen;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Node;
import org.xdv.xpath.XPathValue;
import org.xdv.xpath.XPathValueNodeSet;
import org.xdv.xpath.common.DOMXPathValueNode;

/**
 * XPath node-set - Jaxen W3C DOM kind.
 *
 * @see XPathValueNodeSet
 * @author Tomasz Kokoszka
 */
public class JaxenXPathValueNodeSet implements XPathValueNodeSet {
   
    /** Wrapped node set - {@link List} of {@link org.w3c.dom.Node} */
    private final List nodeSet;
    
    /** 
     * Creates new XPath node set.
     * @param nodeSet node set. List of {@link Node}
     */
    JaxenXPathValueNodeSet(List nodeSet) {
        assert nodeSet != null;
        this.nodeSet = nodeSet;
    }
   
    /**
     * Returns list of XML nodes - order in returned list is the same as order in wrapped node set.
     * @return List of {@link Node}. !=null. If wrapped node-set is empty, then an
     *         empty list is returned. Returned list cannot be modified!
     */
    public List getNodes() {
        return this.nodeSet;
    }
    
    /**
     * Returns list of wrapped XML nodes from this node-set.
     * @return List of {@link DOMXPathValueNode}, !=null. If node-set is empty, then an empty list 
     *         is returned. Returned list can be modified. 
     */            
    public List getNodesAsValues() {
        final List result = new ArrayList(this.nodeSet.size());
        for(Iterator i = this.nodeSet.iterator(); i.hasNext(); ) {
            final Node n = (Node) i.next();
            result.add(new DOMXPathValueNode(n));            
        }
        return result;
    }

    /**
     * Returns the first XML node from this node-set - {@link Node}.
     * @return First XML node (in document order, instance of {@link Node}) 
     *         or <code>null</code> if this node-set is empty.
     */
    public Object getFirstNode() {
        if (this.nodeSet.isEmpty()) {
            return null;
        }
        return this.nodeSet.get(0);
    }
    
    /**
     * Returns <code>true</code> if this node-set contains the specified XML node.
     * @param nodeNative XML node whose presence in this node-set is to be tested
     * @return Value <code>true</code> if and only if this node-set contains the specified XML node
     */
    public boolean containsNode(Object nodeNative) {
        return this.nodeSet.contains(nodeNative);
    }

    public int getLength() {
        return nodeSet.size();
    }
    
    public boolean isEmpty() {
        return nodeSet.isEmpty();
    }

    /**
     * Returns wrapped object - instance of {@link List} of {@link Node}.
     * @return Wrapped object - {@link List} of {@link Node}, !=null
     */
    public Object getNative() {
        return nodeSet;
    }

    /**
     * Returns value type.
     * @return {@link XPathValue#TYPE_NODESET}
     */
    public int getType() {
        return XPathValue.TYPE_NODESET;
    }
        
    public String asString() {
        final StringBuffer sb = new StringBuffer();
        for(Iterator i = nodeSet.iterator(); i.hasNext(); ) {            
            final Node n = (Node) i.next();
            sb.append(DOMXPathValueNode.toString(n));
        }
        return sb.toString();
    }

    public boolean asBoolean() {
        return !nodeSet.isEmpty();
    }

    public double asNumber() {
        try {
            return Double.parseDouble(this.asString());
        } catch (NumberFormatException e) {
            return Double.NaN;
        }           
    }
    
    public String toString() {
        final StringBuffer sb = new StringBuffer();
        for(Iterator i = nodeSet.iterator(); i.hasNext(); ) {            
            final Node n = (Node) i.next();
            sb.append(DOMXPathValueNode.toString(n));
        }
        return sb.toString();
    }
    
}
